@@ -3,7 +3,7 @@ using Mono.Unix;
 using Gtk;
 using Gdk;
 using GLib;
-
+using FSpot.Utils;
 
 namespace FSpot {
 	public class GroupSelector : Fixed {
@@ -39,8 +39,10 @@ namespace FSpot {
 		protected FSpot.GroupAdaptor adaptor;
 		public FSpot.GroupAdaptor Adaptor {
 			set {
-				if (adaptor != null)
+				if (adaptor != null) {
 					adaptor.Changed -= HandleAdaptorChanged;
+					adaptor.Dispose ();
+				}
 
 				adaptor = value;
 				HandleAdaptorChanged (adaptor);
@@ -52,14 +54,14 @@ namespace FSpot {
 				}
 			
 				if (adaptor is TimeAdaptor) {
-					MainWindow.SetTip (left, Catalog.GetString ("More dates"));
-					MainWindow.SetTip (right, Catalog.GetString ("More dates"));
+					MainWindow.ToolTips.SetTip (left, Catalog.GetString ("More dates"), null);
+					MainWindow.ToolTips.SetTip (right, Catalog.GetString ("More dates"), null);
 				} else if (adaptor is DirectoryAdaptor) {
-					MainWindow.SetTip (left, Catalog.GetString ("More directories"));
-					MainWindow.SetTip (right, Catalog.GetString ("More directories"));
+					MainWindow.ToolTips.SetTip (left, Catalog.GetString ("More directories"), null);
+					MainWindow.ToolTips.SetTip (right, Catalog.GetString ("More directories"), null);
 				} else {
-					MainWindow.SetTip (left, Catalog.GetString ("More"));
-					MainWindow.SetTip (right, Catalog.GetString ("More"));
+					MainWindow.ToolTips.SetTip (left, Catalog.GetString ("More"), null);
+					MainWindow.ToolTips.SetTip (right, Catalog.GetString ("More"), null);
 				}
 				
 				adaptor.Changed += HandleAdaptorChanged;
@@ -369,8 +371,10 @@ namespace FSpot {
 			attr.Wclass = WindowClass.InputOnly;
 			attr.EventMask = (int) Events;
 			attr.EventMask |= (int) (EventMask.ButtonPressMask | 
-				EventMask.ButtonReleaseMask | 
-				EventMask.PointerMotionMask);
+						 EventMask.KeyPressMask |
+						 EventMask.KeyReleaseMask |
+						 EventMask.ButtonReleaseMask | 
+						 EventMask.PointerMotionMask);
 				
 			event_window = new Gdk.Window (GdkWindow, attr, (int) (WindowAttributesType.X | WindowAttributesType.Y));
 			event_window.UserData = this.Handle;
@@ -413,13 +417,16 @@ namespace FSpot {
 				bounds.Y = selector.background.Y;
 				bounds.X = selector.BoxX (item);
 				bounds.Width = Math.Max (selector.BoxX (item + 1) - bounds.X, 1);
-				
+
+				if (item < 0 || item > selector.box_counts.Length - 1)
+					return;
+
 				double percent = selector.box_counts [item] / (double) Math.Max (selector.box_count_max, 1);
 				bar = bounds;
 				bar.Height = (int) Math.Ceiling ((bounds.Height - selector.box_top_padding) * percent);
 				bar.Y += bounds.Height - bar.Height - 1;
 				
-				bar.Inflate (-selector.box_spacing, 0);
+				bar.Inflate (- selector.box_spacing, 0);
 			}
 			
 			public Gdk.Rectangle Bounds {
@@ -483,7 +490,7 @@ namespace FSpot {
 					layout.GetPixelSize (out width, out height);
 					
 					Style.PaintLayout (Style, GdkWindow, State, true, area, this,
-						   "GroupSelector:Tick", tick.X, tick.Y + tick.Height, layout); 
+						   "GroupSelector:Tick", tick.X + 3, tick.Y + tick.Height, layout); 
 				}
 			}
 
@@ -494,25 +501,41 @@ namespace FSpot {
 				GdkWindow.DrawRectangle (Style.ForegroundGC (State), true, area);
 			}
 		}
+		
+		protected override bool OnPopupMenu ()
+		{
+			DrawOrderMenu (null);
+			return true;
+		}
 
 		private bool DrawOrderMenu (Gdk.EventButton args)
 		{
 			Gtk.Menu order_menu = new Gtk.Menu();
 			
-			GtkUtil.MakeCheckMenuItem (order_menu, Mono.Posix.Catalog.GetString ("Arrange by _Month"),
+			GtkUtil.MakeCheckMenuItem (order_menu, Catalog.GetString ("Arrange by _Month"),
 					      MainWindow.Toplevel.HandleArrangeByTime, true, (adaptor is TimeAdaptor), true);
 			
-			GtkUtil.MakeCheckMenuItem (order_menu, Mono.Posix.Catalog.GetString ("Arrange by _Folder"),
+			GtkUtil.MakeCheckMenuItem (order_menu, Catalog.GetString ("Arrange by _Folder"),
 					      MainWindow.Toplevel.HandleArrangeByDirectory, true, (adaptor is DirectoryAdaptor), true);
 
 			GtkUtil.MakeMenuSeparator (order_menu);
 
-			GtkUtil.MakeCheckMenuItem (order_menu, Mono.Posix.Catalog.GetString ("_Reverse Order"),
+			GtkUtil.MakeCheckMenuItem (order_menu, Catalog.GetString ("_Reverse Order"),
 					      MainWindow.Toplevel.HandleReverseOrder, true, adaptor.OrderAscending, false);
 
-			order_menu.Popup (null, null, null, args.Button, args.Time);
+			if (adaptor is TimeAdaptor && adaptor.Query.Range != null) {
+				GtkUtil.MakeMenuSeparator (order_menu);
 
-			return base.OnButtonPressEvent (args);
+				GtkUtil.MakeMenuItem (order_menu, Catalog.GetString ("_Clear Date Range"), 
+						MainWindow.Toplevel.HandleClearDateRange);
+			}
+			
+			if (args != null)
+				order_menu.Popup (null, null, null, args.Button, args.Time);
+			else
+				order_menu.Popup (null, null, null, 0, Gtk.Global.CurrentEventTime);
+			
+			return true;
 		}
 
 		public abstract class Manipulator {
@@ -566,6 +589,14 @@ namespace FSpot {
 
 				return true;
 			}
+			
+			protected bool PositionValid (int position)
+			{
+				if (position < 0 || position > selector.box_counts.Length - 1)
+					return false;
+
+				return true;
+			}
 
 			public virtual void UpdateDrag (double x, double y)
 			{
@@ -578,8 +609,7 @@ namespace FSpot {
 						return;
 				}
 
-				System.Console.WriteLine ("scroll {0}, x {1}", scroll, x);
-
+				//System.Console.WriteLine ("scroll {0}, x {1}", scroll, x);
 				DragOffset = (int)x - DragStart.X;
 			}
 
@@ -621,6 +651,9 @@ namespace FSpot {
 
 			public void SetPosition (int position, bool update)
 			{
+				if (! PositionValid (position))
+					return;
+				    
 				Rectangle then = Bounds ();
 				this.position = position;
 				Rectangle now = Bounds ();
@@ -662,12 +695,12 @@ namespace FSpot {
 			public Glass (GroupSelector selector) : base (selector) 
 			{
 				popup_window = new TipWindow ();
-				popup_label = new Gtk.Label ("");
+				popup_label = new Gtk.Label (String.Empty);
 				popup_label.Show ();
 				popup_window.Add (popup_label);
 			}
 
-			private int handle_height = 15;
+			public int handle_height = 15;
 			private int border {
 				get {
 					return selector.box_spacing * 2;
@@ -749,40 +782,44 @@ namespace FSpot {
 
 			public override void Draw (Rectangle area)
 			{
+				if (! PositionValid (Position))
+					return;
+
 				Rectangle inner = InnerBounds ();
 				Rectangle bounds = Bounds ();
 				
-				if (bounds.Intersect (area, out area)) {
-					int i = 0;
-
-					Rectangle box = inner;
-					box.Width -= 1;
-					box.Height -= 1;
-					while (i < border) {
-						box.Inflate (1, 1);
-						
-						selector.Style.BackgroundGC (State).ClipRectangle = area;
-						selector.GdkWindow.DrawRectangle (selector.Style.BackgroundGC (State), 
-										  false, box);
-						i++;
-					}
-
-
-					Style.PaintFlatBox (selector.Style, selector.GdkWindow, State, ShadowType.In, 
-							    area, selector, "glass", bounds.X, inner.Y + inner.Height + border, 
-							    bounds.Width, handle_height);
-
-					Style.PaintHandle (selector.Style, selector.GdkWindow, State, ShadowType.In, 
-							    area, selector, "glass", bounds.X, inner.Y + inner.Height + border, 
-							    bounds.Width, handle_height, Orientation.Horizontal);
+				if (! bounds.Intersect (area, out area))
+				    return;
+				    
+				int i = 0;
+				
+				Rectangle box = inner;
+				box.Width -= 1;
+				box.Height -= 1;
+				while (i < border) {
+					box.Inflate (1, 1);
 					
-					Style.PaintShadow (selector.Style, selector.GdkWindow, State, ShadowType.Out, 
-							   area, selector, "glass", bounds.X, bounds.Y, bounds.Width, bounds.Height);
-
-					Style.PaintShadow (selector.Style, selector.GdkWindow, State, ShadowType.In, 
-							   area, selector, "glass", inner.X, inner.Y, inner.Width, inner.Height);
-
+					selector.Style.BackgroundGC (State).ClipRectangle = area;
+					selector.GdkWindow.DrawRectangle (selector.Style.BackgroundGC (State), 
+									  false, box);
+					i++;
 				}
+				
+				
+				Style.PaintFlatBox (selector.Style, selector.GdkWindow, State, ShadowType.In, 
+						    area, selector, "glass", bounds.X, inner.Y + inner.Height + border, 
+						    bounds.Width, handle_height);
+				
+				Style.PaintHandle (selector.Style, selector.GdkWindow, State, ShadowType.In, 
+						   area, selector, "glass", bounds.X, inner.Y + inner.Height + border, 
+						   bounds.Width, handle_height, Orientation.Horizontal);
+				
+				Style.PaintShadow (selector.Style, selector.GdkWindow, State, ShadowType.Out, 
+						   area, selector, null, bounds.X, bounds.Y, bounds.Width, bounds.Height);
+				
+				Style.PaintShadow (selector.Style, selector.GdkWindow, State, ShadowType.In, 
+						   area, selector, null, inner.X, inner.Y, inner.Width, inner.Height);
+				
 			}
 			
 			public override void PositionChanged ()
@@ -849,10 +886,10 @@ namespace FSpot {
 						    selector, "IconView:Selector", bottom_points, true);
 #else
 				Style.PaintBox (selector.Style, selector.GdkWindow, State, ShadowType.Out, area,
-						selector, "limit", top.X, top.Y, top.Width, top.Height);
+						selector, null, top.X, top.Y, top.Width, top.Height);
 
 				Style.PaintBox (selector.Style, selector.GdkWindow, State, ShadowType.Out, area,
-						selector, "limit", bottom.X, bottom.Y, bottom.Width, bottom.Height);
+						selector, null, bottom.X, bottom.Y, bottom.Width, bottom.Height);
 #endif
 			}
 
@@ -929,7 +966,7 @@ namespace FSpot {
 		protected override void OnSizeRequested (ref Requisition requisition)
 		{
 			requisition.Width = 500;
-			requisition.Height = 70;
+			requisition.Height = (int) (LegendHeight () + glass.handle_height + 3 * border);
 			base.OnSizeRequested (ref requisition);
 		}
 
@@ -999,7 +1036,7 @@ namespace FSpot {
 	
 			Gdk.Rectangle bar = new Rectangle (alloc.X + border, alloc.Y + border,
 							   alloc.Width - 2 *  border,
-							   alloc.Height - 2 * border - legend_height);
+							   alloc.Height - 2 * border - glass.handle_height);
 							   
 		
 			if (left.Allocation.Y != bar.Y || left.Allocation.X != bar.X) {
@@ -1017,10 +1054,10 @@ namespace FSpot {
 						    bar.Width - left.Allocation.Width - right.Allocation.Width, 
 						    bar.Height);
 			
-			legend = new Rectangle (background.X, background.Y + background.Height,
+			legend = new Rectangle (background.X, background.Y,
 						background.Width, legend_height);
 
-			action = background.Union (legend);
+			action = background.Union (glass.Bounds ());
 
 			if (event_window != null)
 				event_window.MoveResize (action.X, action.Y, action.Width, action.Height);
@@ -1030,6 +1067,23 @@ namespace FSpot {
 			UpdateButtons ();
 		}
 
+		public void ResetLimits ()
+		{
+			min_limit.SetPosition(0,false);
+			max_limit.SetPosition(adaptor.Count () - 1, false);
+		}
+
+		public void SetLimitsToDates(DateTime start, DateTime stop)
+		{
+			if (((TimeAdaptor)adaptor).OrderAscending) {
+				min_limit.SetPosition(((TimeAdaptor)adaptor).IndexFromDate(start),false);
+				max_limit.SetPosition(((TimeAdaptor)adaptor).IndexFromDate(stop),false);
+			} else {
+				min_limit.SetPosition(((TimeAdaptor)adaptor).IndexFromDate(stop),false);
+				max_limit.SetPosition(((TimeAdaptor)adaptor).IndexFromDate(start),false);
+			}
+		}
+
 		public GroupSelector () : base () 
 		{
 			SetFlag (WidgetFlags.NoWindow);
@@ -1061,6 +1115,8 @@ namespace FSpot {
 			this.Put (right, 100, 0);
 			left.Show ();
 			right.Show ();
+
+			CanFocus = true;
 			
 			Mode = RangeType.Min;
 			UpdateButtons ();
